<?php
namespace Zeedhi\Framework\DTO;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\HTTP\Kernel;

class Response {

    public const STATUS_SUCCESS = 'S';
    public const STATUS_ERROR = 'E';

    /** @var string */
    protected $status;
    /** @var Response\Message[] */
    protected $messages;
    /** @var Response\Error */
    protected $error;
    /** @var  DataSet[] */
    protected $dataSets;
    /** @var  Response\Method[] */
    protected $methods;
    /** @var  Response\Notification[] */
    protected $notifications;
    /** @var  Response\File */
    protected $fileToDownload;
    /** @var  Next\Response */
    protected $next;
    /** @var  mixed[] */
    protected $metadata;
    /** @var  mixed[] */
    protected $variables;
    /** @var  string */
    protected $language;

    function __construct() {
        $this->status = self::STATUS_SUCCESS;
        $this->methods = [];
        $this->messages = [];
        $this->dataSets = [];
        $this->notifications = [];
        $this->variables = [];
        $this->fileToDownload = null;
        $this->next = null;
        $this->metadata = [];
        $this->language = Kernel::$language;
    }

    /**
     * @return DataSet[]
     */
    public function getDataSets() : array {
        return $this->dataSets;
    }

    /**
     * @param DataSet[]
     */
    public function setDataSets(array $dataSets) {
        $this->dataSets = $dataSets;
    }

    /**
     * @return mixed[]
     */
    public function getMetadata() : array {
        return $this->metadata;
    }

    /**
     * @param mixed[]
     */
    public function setMetadata(array $metadata) {
        $this->metadata = $metadata;
    }

    /**
     * Retrieves the Zeedhi Next Object from the response object.
     * @return Next\Response|null
     */
    public function getZeedhiNext() {
        return $this->next;
    }

    /**
     * Sets the Zeedhi Next to the response object.
     * @param Next\Response
     */
    public function setZeedhiNext(Next\Response $next) {
        $this->next = $next;
    }

    /**
     * @return Response\Error|null
     */
    public function getError() {
        return $this->error;
    }

    /**
     * @return Response\Message[]
     */
    public function getMessages() : array {
        return $this->messages;
    }

    /**
     * @return Response\Method[]
     */
    public function getMethods() : array {
        return $this->methods;
    }

    /**
     * @return string
     */
    public function getStatus() : string {
        return $this->status;
    }

    /**
     * @return Response\Notification[]
     */
    public function getNotifications() : array {
        return $this->notifications;
    }

    /**
     * @return Response\File
     */
    public function getFileToDownload() : Response\File {
        return $this->fileToDownload;
    }

    /**
     * @param Response\Error $error
     */
    public function setError(Response\Error $error) {
        $this->error = $error;
        $this->setStatus(Response::STATUS_ERROR);
    }

    /**
     * @param string $status
     */
    public function setStatus(string $status) {
        $this->status = $status;
    }

    /**
     * @param Response\File $file
     */
    public function setFileToDownload(Response\File $file) {
        $this->fileToDownload = $file;
    }

    /**
     * @param Response\Message
     */
    public function addMessage(Response\Message $message) {
        $this->messages[] = $message;
    }

    /**
     * @param DataSet
     */
    public function addDataSet(DataSet $dataSet) {
        $this->dataSets[] = $dataSet;
    }

    /**
     * @param Response\Method
     */
    public function addMethod(Response\Method $method) {
        $this->methods[] = $method;
    }

    /**
     * @param Response\Notification
     */
    public function addNotification(Response\Notification $notification) {
        $this->notifications[] = $notification;
    }

    /**
     * @return bool
    */
    public function isFileDownloadResponse() : bool {
        return $this->fileToDownload instanceof Response\File;
    }

    /**
     * @param mixed[]
    */
    public function setVariables(array $variables = []) {
        $this->variables = $variables;
    }

    /**
     * @return array
    */
    public function getVariables() : array {
        return $this->variables;
    }

    /**
     * @param string $language
    */
    public function setLanguage(string $language) {
        $this->language = $language;
    }

    /**
     * @param string
    */
    public function getLanguage() : string {
        return $this->language;
    }
}