<?php
namespace tests\Zeedhi\Framework\DataSource\Manager\SQL;

use HumanRelation\Util\EntityManagerFactory;
use Zeedhi\Framework\DataSource\FilterCriteria;

use Doctrine\DBAL\DBALException;

class SQLServerManagerImplTest extends ManagerImplTest {

    public function setUp() {
        if (!extension_loaded('sqlsrv')) {
            $this->markTestSkipped('The SQLServer (sqlsrv) extension is not available.');
        }

        parent::setUp();
    }

    protected function getEntityManager() {
        return EntityManagerFactory::createSqlServerEntityManager();
    }

    /**
     * It's not possible run this test, table DUAL not exists on
     * SQL Server.
     */
     public function testQueryWithoutTable() {}

    public function testFindByRelationsWithNullValues() {
        $filterCriteria = new FilterCriteria("employees");
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $this->assertCount(107, $dataSet->getRows());
    }

    public function testFindWithExtraColumnInDataSourceConfig() {
        $this->expectException(DBALException::class);
        $filterCriteria = new FilterCriteria("invalid_column_countries");
        $filterCriteria->addCondition("COUNTRY_ID", FilterCriteria::EQ, 'BR');
        $this->dataSourceManager->findBy($filterCriteria);
    }

    public function testFindWithQueryInDataSourceConfig() {
        $filterCriteria = new FilterCriteria("employees_with_query");
        $this->parameterBag->set("FIRST_NAME", '%lan%');
        $filterCriteria->addCondition("SALARY", FilterCriteria::LT, 5000);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $this->assertCount(1, $dataSet->getRows());
    }

    public function testFullSearch() {
        $expectedCountryIds = array('AU   ', 'US   ');
        $filterCriteria = new FilterCriteria("countries");
        $filterCriteria->addCondition("*", FilterCriteria::LIKE_ALL, '%US%');
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        $this->assertCount(2, $rows);
        foreach($rows as $row){
            $this->assertContains($row['COUNTRY_ID'], $expectedCountryIds);
        }
    }

    public function testFullSearchWithColumnList() {
        $expectedCountryIds = array('AU   ', 'US   ');
        $filterCriteria = new FilterCriteria("countries");
        $filterCriteria->addCondition("COUNTRY_ID|COUNTRY_NAME", FilterCriteria::LIKE_ALL, '%US%');
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        $this->assertCount(2, $rows);
        foreach($rows as $row){
            $this->assertContains($row['COUNTRY_ID'], $expectedCountryIds);
        }
    }

    public function testFindByInQueryDataSource() {
        $expectedRow = array(
            'COUNTRY_ID'   => 'BR   ',
            'COUNTRY_NAME' => 'Brazil',
            'REGION_ID'    => 2,
            'REGION_NAME'  => 'Americas',
        );
        $filterCriteria = new FilterCriteria("countries_with_regions");
        $filterCriteria->addCondition("COUNTRY_ID", FilterCriteria::EQ, 'BR');
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $this->assertCount(1, $dataSet->getRows(), "Should return 1 rows");
        $row = @array_pop($dataSet->getRows());
        foreach($expectedRow as $columnName => $columnValue) {
            $this->assertArrayHasKey($columnName, $row);
            $this->assertEquals($columnValue, $row[$columnName]);
        }
    }

}